package com.brilliance.swift.util;

import com.brilliance.swift.exception.IORuntimeException;
import org.apache.commons.io.IOUtils;

import java.io.File;
import java.io.FileFilter;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.List;

public class FileUtil {
    public static final String CLASSPATH_URL_PREFIX = "classpath:";
    public static final String URL_PROTOCOL_FILE = "file";

    public static URL getURL(String resourceLocation) {
        if (resourceLocation.startsWith(CLASSPATH_URL_PREFIX)) {
            String path = resourceLocation.substring(CLASSPATH_URL_PREFIX.length());
            ClassLoader cl = ClassUtil.getDefaultClassLoader();
            URL url = (cl != null ? cl.getResource(path) : ClassLoader.getSystemResource(path));
            if (url == null) {
                String description = "class path resource [" + path + "]";
                throw new IllegalArgumentException(description +
                        " cannot be resolved to URL because it does not exist");
            }
            return url;
        }
        try {
            if (isAbsolutePath(resourceLocation)) {
                //如果是绝对路径直接返回
                return new File(resourceLocation).toURI().toURL();
            }
            //重复:考虑有的没有加classpath:前缀
            ClassLoader cl = ClassUtil.getDefaultClassLoader();
            URL url = (cl != null ? cl.getResource(resourceLocation) : ClassLoader.getSystemResource(resourceLocation));
            if (url == null) {
                //视为url协议
                return new URL(resourceLocation);
            }
            return url;
        } catch (MalformedURLException ex) {
            // 视为文件路径
            try {
                return new File(resourceLocation).toURI().toURL();
            } catch (MalformedURLException ex2) {
                throw new IllegalArgumentException("Resource location [" + resourceLocation +
                        "] is neither a URL not a well-formed file path");
            }
        }
    }

    public static File getFile(String path) {
        return getFile(getURL(path));
    }

    public static File getFile(String rootPath, String childPath) {
        File rootFile = getFile(rootPath);
        return new File(rootFile, childPath);
    }

    public static File getFile(URL resourceUrl) {
        if (!URL_PROTOCOL_FILE.equals(resourceUrl.getProtocol())) {
            throw new IllegalArgumentException("url cannot be resolved to absolute file path " +
                    "because it does not reside in the file system: " + resourceUrl);
        }
        return new File(toURI(resourceUrl).getSchemeSpecificPart());
    }

    public static URI toURI(URL url) {
        return toURI(url.toString());
    }

    public static URI toURI(String location) {
        try {
            return new URI(location.replace(" ", "%20"));
        } catch (Exception e) {
            throw new IllegalArgumentException("URI转换异常:" + e.getMessage(), e);
        }

    }

    public static List<File> loopFiles(File file, FileFilter fileFilter) {
        List<File> fileList = new ArrayList();
        if (null != file && file.exists()) {
            if (file.isDirectory()) {
                File[] subFiles = file.listFiles();
                if (!ObjectUtil.isEmpty(subFiles)) {
                    for (int i = 0; i < subFiles.length; ++i) {
                        File tmp = subFiles[i];
                        fileList.addAll(loopFiles(tmp, fileFilter));
                    }
                }
            } else if (null == fileFilter || fileFilter.accept(file)) {
                fileList.add(file);
            }

            return fileList;
        } else {
            return fileList;
        }
    }

    /**
     * 获取文件或者目录名，如果是文件名则不带扩展名
     */
    public static String getFileName(File file) {
        if (null == file) {
            return null;
        } else {
            return file.isDirectory() ? file.getName() : preName(file.getName());
        }
    }

    public static String getSuffix(File file) {
        if (null == file) {
            return null;
        } else {
            return file.isDirectory() ? null : extName(file.getName());
        }
    }

    public static String extName(String fileName) {
        if (fileName == null) {
            return null;
        } else {
            int index = fileName.lastIndexOf(".");
            if (index == -1) {
                return "";
            } else {
                String ext = fileName.substring(index + 1);
                return StringUtil.containsAny(ext, new char[]{'/', '\\'}) ? "" : ext;
            }
        }
    }

    public static String preName(String fileName) {
        if (fileName == null) {
            return null;
        } else {
            int index = fileName.lastIndexOf(".");
            if (index == -1) {
                return "";
            } else {
                return fileName.substring(0, index);
            }
        }
    }

    public static File writeString(String content, String path) {
        return writeString(content, path, Charset.defaultCharset());
    }

    public static File writeString(String content, String path, Charset charset) {
        return writeString(content, touch(path), charset, false);
    }

    public static File writeString(String content, File file, Charset charset, boolean append) {
        FileOutputStream out = null;
        try {
            out = openOutputStream(file, append);
            IOUtils.write(content, out, charset);
            out.close();
        } catch (Exception e) {
            throw new IORuntimeException("Write String to File exception",e);
        } finally {
            IOUtils.closeQuietly(out);
        }
        return file;
    }

    public static FileOutputStream openOutputStream(File file, boolean append) {
        if (file.exists()) {
            if (file.isDirectory()) {
                throw new IORuntimeException("File '" + file + "' exists but is a directory");
            }

            if (!file.canWrite()) {
                throw new IORuntimeException("File '" + file + "' cannot be written to");
            }
        } else {
            File parent = file.getParentFile();
            if (parent != null && !parent.mkdirs() && !parent.isDirectory()) {
                throw new IORuntimeException("Directory '" + parent + "' could not be created");
            }
        }
        try {
            return new FileOutputStream(file, append);
        } catch (FileNotFoundException e) {
            throw new IORuntimeException("File Not Found Exception", e);
        }
    }

    public static File touch(String fullFilePath) {
        return fullFilePath == null ? null : touch(getFile(fullFilePath));
    }

    public static File touch(File file) throws IORuntimeException {
        if (null == file) {
            return null;
        } else {
            if (!file.exists()) {
                mkParentDirs(file);
                try {
                    file.createNewFile();
                } catch (Exception ex) {
                    throw new IORuntimeException("create new file exception", ex);
                }
            }

            return file;
        }
    }

    public static File mkParentDirs(File file) {
        File parentFile = file.getParentFile();
        if (null != parentFile && !parentFile.exists()) {
            parentFile.mkdirs();
        }

        return parentFile;
    }

    public static boolean isAbsolutePath(String path) {
        if (StringUtil.isEmpty(path)) {
            return false;
        } else {
            return '/' == path.charAt(0) || path.matches("^[a-zA-Z]:([/\\\\].*)?");
        }
    }

}
